package ru.dbotthepony.mc.otm.menu.matter

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.block.entity.matter.MatterReconstructorBlockEntity
import ru.dbotthepony.mc.otm.menu.MatteryPoweredMenu
import ru.dbotthepony.mc.otm.menu.MatterySlot
import ru.dbotthepony.mc.otm.menu.input.EnergyPlayerInput
import ru.dbotthepony.mc.otm.menu.input.EnumInputWithFeedback
import ru.dbotthepony.mc.otm.menu.input.ItemHandlerPlayerInput
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class MatterReconstructorMenu(
	containerId: Int,
	inventory: Inventory,
	tile: MatterReconstructorBlockEntity? = null
) : MatteryPoweredMenu(MMenus.ITEM_REPAIER, containerId, inventory, tile) {
	val matterWidget = LevelGaugeWidget(this, tile?.matter)
	val slot = object : MatterySlot(tile?.repairContainer ?: SimpleContainer(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return itemStack.isRepairable && itemStack.isDamaged && super.mayPlace(itemStack)
		}
	}

	val equipment = makeEquipmentSlots(mapMoveToExternal = true)
	val progress: ProgressGaugeWidget

	val redstoneControl = EnumInputWithFeedback<RedstoneSetting>(this)
	val energyConfig = EnergyPlayerInput(this)
	val itemConfig = ItemHandlerPlayerInput(this)

	init {
		addStorageSlot(slot)
		addInventorySlots()

		if (tile != null) {
			redstoneControl.with(tile.redstoneControl::redstoneSetting)
			itemConfig.configure(tile.itemConfig)
			energyConfig.configure(tile.energyConfig)
		}

		if (tile == null) {
			progress = ProgressGaugeWidget(this)
		} else {
			progress = ProgressGaugeWidget(this, tile::visualProgress, tile::isUnableToProcess)
		}
	}
}
