package ru.dbotthepony.mc.otm.menu.tech

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.RedstoneSetting
import ru.dbotthepony.mc.otm.block.entity.tech.EnergyServoBlockEntity
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatterySlot
import ru.dbotthepony.mc.otm.menu.input.EnergyPlayerInput
import ru.dbotthepony.mc.otm.menu.input.EnumInputWithFeedback
import ru.dbotthepony.mc.otm.menu.input.ItemHandlerPlayerInput
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class EnergyServoMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: EnergyServoBlockEntity? = null
) : MatteryMenu(MMenus.ENERGY_SERVO, p_38852_, inventory, tile) {
	val dischargeSlot = object : MatterySlot(tile?.discharge ?: SimpleContainer(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return super.mayPlace(itemStack) && (itemStack.energy?.canExtract() ?: false)
		}
	}

	val chargeSlot = object : MatterySlot(tile?.charge ?: SimpleContainer(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return super.mayPlace(itemStack) && (itemStack.energy?.canReceive() ?: false)
		}
	}

	val equipment = makeEquipmentSlots(mapMoveToExternal = true)

	val powerGauge = LevelGaugeWidget(this, tile?.energy)
	val itemConfig = ItemHandlerPlayerInput(this)
	val energyConfig = EnergyPlayerInput(this, true, true)
	val redstone = EnumInputWithFeedback(this, RedstoneSetting::class.java)

	init {
		if (tile != null) {
			redstone.with(tile.redstoneControl::redstoneSetting)
			itemConfig.configure(tile.itemConfig)
			energyConfig.configure(tile.energyConfig)
		}
	}

	init {
		// charge then discharge for charging to take priority over discharging
		addStorageSlot(chargeSlot)
		addStorageSlot(dischargeSlot)

		addInventorySlots()
	}
}
